@@ -91,9 +91,12 @@ bool OptionsParser::parse()
         // probably a file or something
         m_pmPrivate->arguments << m_currentArg;
     }
+    if (PluginManager::testRunRequested()) {
+        m_isDependencyRefreshNeeded = true;
+        forceDisableAllPluginsExceptTestedAndForceEnabled();
+    }
     if (m_isDependencyRefreshNeeded)
-        m_pmPrivate->resolveDependencies();
-    m_pmPrivate->enableOnlyTestedSpecs();
+        m_pmPrivate->enableDependenciesIndirectly();
     return !m_hasError;
 }
 
@@ -112,10 +115,10 @@ bool OptionsParser::checkForTestOptions()
     if (m_currentArg == QLatin1String(TEST_OPTION)) {
         if (nextToken(RequiredToken)) {
             if (m_currentArg == QLatin1String("all")) {
-                m_pmPrivate->testSpecs =
-                        Utils::transform(m_pmPrivate->loadQueue(), [](PluginSpec *spec) {
-                            return PluginManagerPrivate::TestSpec(spec);
-                        });
+                m_pmPrivate->testSpecs
+                    = Utils::transform<std::vector>(m_pmPrivate->loadQueue(), [](PluginSpec *spec) {
+                          return PluginManagerPrivate::TestSpec(spec);
+                      });
             } else {
                 QStringList args = m_currentArg.split(QLatin1Char(','));
                 const QString pluginName = args.takeFirst();
@@ -126,7 +129,7 @@ bool OptionsParser::checkForTestOptions()
                                                                          "The plugin \"%1\" is specified twice for testing.").arg(pluginName);
                         m_hasError = true;
                     } else {
-                        m_pmPrivate->testSpecs.append(PluginManagerPrivate::TestSpec(spec, args));
+                        m_pmPrivate->testSpecs.emplace_back(spec, args);
                     }
                 } else  {
                     if (m_errorString)
@@ -260,6 +263,16 @@ bool OptionsParser::checkForUnknownOption()
     return true;
 }
 
+void OptionsParser::forceDisableAllPluginsExceptTestedAndForceEnabled()
+{
+    for (const PluginManagerPrivate::TestSpec &testSpec : m_pmPrivate->testSpecs)
+        testSpec.pluginSpec->d->setForceEnabled(true);
+    for (PluginSpec *spec : qAsConst(m_pmPrivate->pluginSpecs)) {
+        if (!spec->isForceEnabled() && !spec->isRequired())
+            spec->d->setForceDisabled(true);
+    }
+}
+
 bool OptionsParser::nextToken(OptionsParser::TokenType type)
 {
     if (m_it == m_end) {
