@@ -24,6 +24,7 @@
 ****************************************************************************/
 
 #include "mainwindow.h"
+
 #include "icore.h"
 #include "jsexpander.h"
 #include "toolsettings.h"
@@ -31,7 +32,6 @@
 #include "fancytabwidget.h"
 #include "documentmanager.h"
 #include "generalsettings.h"
-#include "helpmanager.h"
 #include "idocumentfactory.h"
 #include "messagemanager.h"
 #include "modemanager.h"
@@ -100,26 +100,26 @@ namespace Internal {
 
 enum { debugMainWindow = 0 };
 
-MainWindow::MainWindow() :
-    AppMainWindow(),
-    m_coreImpl(new ICore(this)),
-    m_lowPrioAdditionalContexts(Constants::C_GLOBAL),
-    m_settingsDatabase(new SettingsDatabase(QFileInfo(PluginManager::settings()->fileName()).path(),
-                                            QLatin1String(Constants::IDE_CASED_ID),
-                                            this)),
-    m_progressManager(new ProgressManagerPrivate),
-    m_jsExpander(new JsExpander),
-    m_vcsManager(new VcsManager),
-    m_helpManager(new HelpManager),
-    m_modeStack(new FancyTabWidget(this)),
-    m_generalSettings(new GeneralSettings),
-    m_systemSettings(new SystemSettings),
-    m_shortcutSettings(new ShortcutSettings),
-    m_toolSettings(new ToolSettings),
-    m_mimeTypeSettings(new MimeTypeSettings),
-    m_systemEditor(new SystemEditor),
-    m_toggleLeftSideBarButton(new QToolButton),
-    m_toggleRightSideBarButton(new QToolButton)
+MainWindow::MainWindow()
+    : AppMainWindow()
+    , m_coreImpl(new ICore(this))
+    , m_lowPrioAdditionalContexts(Constants::C_GLOBAL)
+    , m_settingsDatabase(
+          new SettingsDatabase(QFileInfo(PluginManager::settings()->fileName()).path(),
+                               QLatin1String(Constants::IDE_CASED_ID),
+                               this))
+    , m_progressManager(new ProgressManagerPrivate)
+    , m_jsExpander(JsExpander::createGlobalJsExpander())
+    , m_vcsManager(new VcsManager)
+    , m_modeStack(new FancyTabWidget(this))
+    , m_generalSettings(new GeneralSettings)
+    , m_systemSettings(new SystemSettings)
+    , m_shortcutSettings(new ShortcutSettings)
+    , m_toolSettings(new ToolSettings)
+    , m_mimeTypeSettings(new MimeTypeSettings)
+    , m_systemEditor(new SystemEditor)
+    , m_toggleLeftSideBarButton(new QToolButton)
+    , m_toggleRightSideBarButton(new QToolButton)
 {
     (void) new DocumentManager(this);
 
@@ -131,6 +131,7 @@ MainWindow::MainWindow() :
     QCoreApplication::setApplicationName(QLatin1String(Constants::IDE_CASED_ID));
     QCoreApplication::setApplicationVersion(QLatin1String(Constants::IDE_VERSION_LONG));
     QCoreApplication::setOrganizationName(QLatin1String(Constants::IDE_SETTINGSVARIANT_STR));
+    QGuiApplication::setApplicationDisplayName(QLatin1String(Constants::IDE_DISPLAY_NAME));
     QString baseName = QApplication::style()->objectName();
     // Sometimes we get the standard windows 95 style as a fallback
     if (HostOsInfo::isAnyUnixHost() && !HostOsInfo::isMacHost()
@@ -149,6 +150,8 @@ MainWindow::MainWindow() :
     }
 
     QApplication::setStyle(new ManhattanStyle(baseName));
+    m_generalSettings->setShowShortcutsInContextMenu(
+        m_generalSettings->showShortcutsInContextMenu());
 
     setDockNestingEnabled(true);
 
@@ -192,12 +195,25 @@ MainWindow::MainWindow() :
     statusBar()->setProperty("p_styled", true);
 
     auto dropSupport = new DropSupport(this, [](QDropEvent *event, DropSupport *) {
-        return event->source() == 0; // only accept drops from the "outside" (e.g. file manager)
+        return event->source() == nullptr; // only accept drops from the "outside" (e.g. file manager)
     });
     connect(dropSupport, &DropSupport::filesDropped,
             this, &MainWindow::openDroppedFiles);
 }
 
+// Edit View 3D needs to know when the main windows's state or activation change
+void MainWindow::changeEvent(QEvent *event)
+{
+    if (event->type() == QEvent::WindowStateChange) {
+        emit m_coreImpl->windowStateChanged(m_previousWindowStates, windowState());
+        m_previousWindowStates = windowState();
+    } else if (event->type() == QEvent::ActivationChange) {
+        auto lastChild = qobject_cast<QWidget *>(children().last());
+        bool hasPopup = lastChild && lastChild->isActiveWindow();
+        emit m_coreImpl->windowActivationChanged(isActiveWindow(), hasPopup);
+    }
+}
+
 NavigationWidget *MainWindow::navigationWidget(Side side) const
 {
     return side == Side::Left ? m_leftNavigationWidget : m_rightNavigationWidget;
@@ -281,8 +297,6 @@ MainWindow::~MainWindow()
     delete m_modeManager;
     m_modeManager = nullptr;
 
-    delete m_helpManager;
-    m_helpManager = nullptr;
     delete m_jsExpander;
     m_jsExpander = nullptr;
 }
@@ -324,7 +338,7 @@ void MainWindow::closeEvent(QCloseEvent *event)
         return;
     }
 
-    ICore::saveSettings();
+    ICore::saveSettings(ICore::MainWindowClosing);
 
     // Save opened files
     if (!DocumentManager::saveAllModifiedDocuments()) {
@@ -359,7 +373,7 @@ void MainWindow::openDroppedFiles(const QList<DropSupport::FileSpec> &files)
 
 IContext *MainWindow::currentContextObject() const
 {
-    return m_activeContext.isEmpty() ? 0 : m_activeContext.first();
+    return m_activeContext.isEmpty() ? nullptr : m_activeContext.first();
 }
 
 QStatusBar *MainWindow::statusBar() const
@@ -367,6 +381,11 @@ QStatusBar *MainWindow::statusBar() const
     return m_modeStack->statusBar();
 }
 
+InfoBar *MainWindow::infoBar() const
+{
+    return m_modeStack->infoBar();
+}
+
 void MainWindow::registerDefaultContainers()
 {
     ActionContainer *menubar = ActionManager::createMenuBar(Constants::MENU_BAR);
@@ -431,6 +450,16 @@ void MainWindow::registerDefaultContainers()
     ac->appendGroup(Constants::G_HELP_SUPPORT);
     ac->appendGroup(Constants::G_HELP_ABOUT);
     ac->appendGroup(Constants::G_HELP_UPDATES);
+
+    // macOS touch bar
+    ac = ActionManager::createTouchBar(Constants::TOUCH_BAR,
+                                       QIcon(),
+                                       "Main TouchBar" /*never visible*/);
+    ac->appendGroup(Constants::G_TOUCHBAR_HELP);
+    ac->appendGroup(Constants::G_TOUCHBAR_EDITOR);
+    ac->appendGroup(Constants::G_TOUCHBAR_NAVIGATION);
+    ac->appendGroup(Constants::G_TOUCHBAR_OTHER);
+    ac->touchBar()->setApplicationTouchBar();
 }
 
 void MainWindow::registerDefaultActions()
@@ -795,15 +824,10 @@ void MainWindow::openFile()
 static IDocumentFactory *findDocumentFactory(const QList<IDocumentFactory*> &fileFactories,
                                      const QFileInfo &fi)
 {
-    const MimeType mt = Utils::mimeTypeForFile(fi);
-    if (mt.isValid()) {
-        const QString typeName = mt.name();
-        foreach (IDocumentFactory *factory, fileFactories) {
-            if (factory->mimeTypes().contains(typeName))
-                return factory;
-        }
-    }
-    return 0;
+    const QString typeName = Utils::mimeTypeForFile(fi).name();
+    return Utils::findOrDefault(fileFactories, [typeName](IDocumentFactory *f) {
+        return f->mimeTypes().contains(typeName);
+    });
 }
 
 /*! Either opens \a fileNames with editors or loads a project.
@@ -845,6 +869,8 @@ IDocument *MainWindow::openFiles(const QStringList &fileNames,
             QFlags<EditorManager::OpenEditorFlag> emFlags;
             if (flags & ICore::CanContainLineAndColumnNumbers)
                 emFlags |=  EditorManager::CanContainLineAndColumnNumber;
+            if (flags & ICore::SwitchSplitIfAlreadyVisible)
+                emFlags |= EditorManager::SwitchSplitIfAlreadyVisible;
             IEditor *editor = EditorManager::openEditor(absoluteFilePath, Id(), emFlags);
             if (!editor) {
                 if (flags & ICore::StopOnLoadFail)
@@ -953,7 +979,7 @@ void MainWindow::updateContextObject(const QList<IContext *> &context)
     if (debugMainWindow) {
         qDebug() << "new context objects =" << context;
         foreach (IContext *c, context)
-            qDebug() << (c ? c->widget() : 0) << (c ? c->widget()->metaObject()->className() : 0);
+            qDebug() << (c ? c->widget() : nullptr) << (c ? c->widget()->metaObject()->className() : nullptr);
     }
 }
 
@@ -1093,8 +1119,7 @@ void MainWindow::updateContext()
     contexts.add(m_lowPrioAdditionalContexts);
 
     Context uniquecontexts;
-    for (int i = 0; i < contexts.size(); ++i) {
-        const Id id = contexts.at(i);
+    for (const Id &id : qAsConst(contexts)) {
         if (!uniquecontexts.contains(id))
             uniquecontexts.add(id);
     }
@@ -1152,7 +1177,7 @@ void MainWindow::destroyVersionDialog()
 {
     if (m_versionDialog) {
         m_versionDialog->deleteLater();
-        m_versionDialog = 0;
+        m_versionDialog = nullptr;
     }
 }
 
