@@ -1828,7 +1828,8 @@ void DebuggerEngine::setState(DebuggerState state, bool forced)
 
     if (state == EngineRunRequested) {
         emit engineStarted();
-        d->m_perspective->select();
+        if (d->m_perspective)
+            d->m_perspective->select();
     }
 
     showMessage(msg, LogDebug);
@@ -2287,6 +2288,12 @@ void DebuggerEngine::openDisassemblerView(const Location &location)
 void DebuggerEngine::raiseWatchersWindow()
 {
     if (d->m_watchersView && d->m_watchersWindow) {
+        auto currentPerspective = DebuggerMainWindow::currentPerspective();
+        QTC_ASSERT(currentPerspective, return);
+        // if a companion engine has taken over - do not raise the watchers
+        if (currentPerspective->name() != d->m_engine->displayName())
+            return;
+
         if (auto dock = qobject_cast<QDockWidget *>(d->m_watchersWindow->parentWidget())) {
             if (QAction *act = dock->toggleViewAction()) {
                 if (!act->isChecked())
