@@ -137,7 +137,7 @@ void SelectRemoteFileDialog::attachToDevice(Kit *k)
 {
     m_buttonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
     QTC_ASSERT(k, return);
-    IDevice::ConstPtr device = DeviceKitInformation::device(k);
+    IDevice::ConstPtr device = DeviceKitAspect::device(k);
     QTC_ASSERT(device, return);
     SshConnectionParameters sshParams = device->sshParameters();
     m_fileSystemModel.setSshConnection(sshParams);
@@ -207,7 +207,7 @@ public:
 
     QCheckBox *forceLocalCheckBox;
     QLabel *forceLocalLabel;
-    PathChooser *localExecFileName;
+    PathChooser *symbolFileName;
     PathChooser *localCoreFileName;
     QLineEdit *remoteCoreFileName;
     QPushButton *selectRemoteCoreButton;
@@ -220,11 +220,11 @@ public:
     {
         bool isValid() const
         {
-            return validKit && validLocalExecFilename && validCoreFilename;
+            return validKit && validSymbolFilename && validCoreFilename;
         }
 
         bool validKit;
-        bool validLocalExecFilename;
+        bool validSymbolFilename;
         bool validCoreFilename;
         bool localCoreFile;
         bool localKit;
@@ -235,7 +235,7 @@ public:
         State st;
         st.localCoreFile = p.useLocalCoreFile();
         st.validKit = (kitChooser->currentKit() != nullptr);
-        st.validLocalExecFilename = localExecFileName->isValid();
+        st.validSymbolFilename = symbolFileName->isValid();
 
         if (st.localCoreFile)
             st.validCoreFilename = localCoreFileName->isValid();
@@ -258,7 +258,8 @@ AttachCoreDialog::AttachCoreDialog(QWidget *parent)
     d->buttonBox->button(QDialogButtonBox::Ok)->setDefault(true);
     d->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
 
-    d->kitChooser = new DebuggerKitChooser(DebuggerKitChooser::AnyDebugging, this);
+    d->kitChooser = new KitChooser(this);
+    d->kitChooser->setShowIcons(true);
     d->kitChooser->populate();
 
     d->forceLocalCheckBox = new QCheckBox(this);
@@ -270,17 +271,21 @@ AttachCoreDialog::AttachCoreDialog(QWidget *parent)
     d->selectRemoteCoreButton = new QPushButton(PathChooser::browseButtonLabel(), this);
 
     d->localCoreFileName = new PathChooser(this);
-    d->localCoreFileName->setHistoryCompleter(QLatin1String("Debugger.CoreFile.History"));
+    d->localCoreFileName->setHistoryCompleter("Debugger.CoreFile.History");
     d->localCoreFileName->setExpectedKind(PathChooser::File);
     d->localCoreFileName->setPromptDialogTitle(tr("Select Core File"));
 
-    d->localExecFileName = new PathChooser(this);
-    d->localExecFileName->setHistoryCompleter(QLatin1String("LocalExecutable"));
-    d->localExecFileName->setExpectedKind(PathChooser::File);
-    d->localExecFileName->setPromptDialogTitle(tr("Select Executable"));
+    d->symbolFileName = new PathChooser(this);
+    d->symbolFileName->setHistoryCompleter("LocalExecutable");
+    d->symbolFileName->setExpectedKind(PathChooser::File);
+    d->symbolFileName->setPromptDialogTitle(tr("Select Executable or Symbol File"));
+    d->symbolFileName->setToolTip(
+        tr("Select a file containing debug information corresponding to the core file. "
+           "Typically, this is the executable or a *.debug file if the debug "
+           "information is stored separately from the executable."));
 
     d->overrideStartScriptFileName = new PathChooser(this);
-    d->overrideStartScriptFileName->setHistoryCompleter(QLatin1String("Debugger.StartupScript.History"));
+    d->overrideStartScriptFileName->setHistoryCompleter("Debugger.StartupScript.History");
     d->overrideStartScriptFileName->setExpectedKind(PathChooser::File);
     d->overrideStartScriptFileName->setPromptDialogTitle(tr("Select Startup Script"));
 
@@ -296,7 +301,7 @@ AttachCoreDialog::AttachCoreDialog(QWidget *parent)
     formLayout->addRow(tr("Kit:"), d->kitChooser);
     formLayout->addRow(d->forceLocalLabel, d->forceLocalCheckBox);
     formLayout->addRow(tr("Core file:"), coreLayout);
-    formLayout->addRow(tr("&Executable:"), d->localExecFileName);
+    formLayout->addRow(tr("&Executable or symbol file:"), d->symbolFileName);
     formLayout->addRow(tr("Override &start script:"), d->overrideStartScriptFileName);
 
     auto line = new QFrame(this);
@@ -321,7 +326,7 @@ int AttachCoreDialog::exec()
 {
     connect(d->selectRemoteCoreButton, &QAbstractButton::clicked, this, &AttachCoreDialog::selectRemoteCoreFile);
     connect(d->remoteCoreFileName, &QLineEdit::textChanged, this, &AttachCoreDialog::coreFileChanged);
-    connect(d->localExecFileName, &PathChooser::rawPathChanged, this, &AttachCoreDialog::changed);
+    connect(d->symbolFileName, &PathChooser::rawPathChanged, this, &AttachCoreDialog::changed);
     connect(d->localCoreFileName, &PathChooser::rawPathChanged, this, &AttachCoreDialog::coreFileChanged);
     connect(d->forceLocalCheckBox, &QCheckBox::stateChanged, this, &AttachCoreDialog::changed);
     connect(d->kitChooser, &KitChooser::currentIndexChanged, this, &AttachCoreDialog::changed);
@@ -337,8 +342,8 @@ int AttachCoreDialog::exec()
             d->localCoreFileName->setFocus();
         else
             d->remoteCoreFileName->setFocus();
-    } else if (!st.validLocalExecFilename) {
-        d->localExecFileName->setFocus();
+    } else if (!st.validSymbolFilename) {
+        d->symbolFileName->setFocus();
     }
 
     return QDialog::exec();
@@ -348,7 +353,7 @@ bool AttachCoreDialog::isLocalKit() const
 {
     Kit *k = d->kitChooser->currentKit();
     QTC_ASSERT(k, return false);
-    IDevice::ConstPtr device = DeviceKitInformation::device(k);
+    IDevice::ConstPtr device = DeviceKitAspect::device(k);
     QTC_ASSERT(device, return false);
     return device->type() == ProjectExplorer::Constants::DESKTOP_DEVICE_TYPE;
 }
@@ -363,12 +368,12 @@ void AttachCoreDialog::coreFileChanged(const QString &core)
     if (!HostOsInfo::isWindowsHost() && QFile::exists(core)) {
         Kit *k = d->kitChooser->currentKit();
         QTC_ASSERT(k, return);
-        Runnable debugger = DebuggerKitInformation::runnable(k);
+        Runnable debugger = DebuggerKitAspect::runnable(k);
         CoreInfo cinfo = CoreInfo::readExecutableNameFromCore(debugger, core);
         if (!cinfo.foundExecutableName.isEmpty())
-            d->localExecFileName->setFileName(FileName::fromString(cinfo.foundExecutableName));
-        else if (!d->localExecFileName->isValid() && !cinfo.rawStringFromCore.isEmpty())
-            d->localExecFileName->setFileName(FileName::fromString(cinfo.rawStringFromCore));
+            d->symbolFileName->setFileName(FilePath::fromString(cinfo.foundExecutableName));
+        else if (!d->symbolFileName->isValid() && !cinfo.rawStringFromCore.isEmpty())
+            d->symbolFileName->setFileName(FilePath::fromString(cinfo.rawStringFromCore));
     }
     changed();
 }
@@ -411,14 +416,14 @@ QString AttachCoreDialog::localCoreFile() const
     return d->localCoreFileName->path();
 }
 
-QString AttachCoreDialog::localExecutableFile() const
+FilePath AttachCoreDialog::symbolFile() const
 {
-    return d->localExecFileName->path();
+    return d->symbolFileName->fileName();
 }
 
-void AttachCoreDialog::setLocalExecutableFile(const QString &fileName)
+void AttachCoreDialog::setSymbolFile(const QString &symbolFileName)
 {
-    d->localExecFileName->setPath(fileName);
+    d->symbolFileName->setPath(symbolFileName);
 }
 
 void AttachCoreDialog::setLocalCoreFile(const QString &fileName)
