@@ -23,34 +23,45 @@
 **
 ****************************************************************************/
 
+#include "projectconfigurationmodel.h"
 #include "target.h"
 
+#include "buildconfiguration.h"
 #include "buildinfo.h"
+#include "buildmanager.h"
 #include "buildtargetinfo.h"
+#include "deployconfiguration.h"
 #include "deploymentdata.h"
+#include "devicesupport/devicemanager.h"
 #include "kit.h"
 #include "kitinformation.h"
 #include "kitmanager.h"
-#include "buildconfiguration.h"
-#include "deployconfiguration.h"
 #include "project.h"
+#include "projectexplorer.h"
+#include "projectexplorericons.h"
+#include "projectexplorersettings.h"
 #include "runconfiguration.h"
 #include "session.h"
 
-#include <limits>
 #include <coreplugin/coreconstants.h>
-#include <projectexplorer/buildmanager.h>
-#include <projectexplorer/devicesupport/devicemanager.h>
+
 #include <extensionsystem/pluginmanager.h>
-#include <projectexplorer/projectexplorer.h>
-#include <utils/qtcassert.h>
+
 #include <utils/algorithm.h>
+#include <utils/macroexpander.h>
+#include <utils/qtcassert.h>
+#include <utils/stringutils.h>
 
 #include <QDebug>
 #include <QIcon>
 #include <QPainter>
 
-namespace {
+#include <limits>
+
+using namespace Utils;
+
+namespace ProjectExplorer {
+
 const char ACTIVE_BC_KEY[] = "ProjectExplorer.Target.ActiveBuildConfiguration";
 const char BC_KEY_PREFIX[] = "ProjectExplorer.Target.BuildConfiguration.";
 const char BC_COUNT_KEY[] = "ProjectExplorer.Target.BuildConfigurationCount";
@@ -64,64 +75,63 @@ const char RC_KEY_PREFIX[] = "ProjectExplorer.Target.RunConfiguration.";
 const char RC_COUNT_KEY[] = "ProjectExplorer.Target.RunConfigurationCount";
 const char PLUGIN_SETTINGS_KEY[] = "ProjectExplorer.Target.PluginSettings";
 
-} // namespace
+static QString formatDeviceInfo(const ProjectExplorer::IDevice::DeviceInfo &input)
+{
+    const QStringList lines
+            = Utils::transform(input, [](const ProjectExplorer::IDevice::DeviceInfoItem &i) {
+        return QString::fromLatin1("<b>%1:</b> %2").arg(i.key, i.value);
+    });
+    return lines.join(QLatin1String("<br>"));
+}
 
 // -------------------------------------------------------------------------
 // Target
 // -------------------------------------------------------------------------
 
-namespace ProjectExplorer {
-
 class TargetPrivate
 {
 public:
-    TargetPrivate(Kit *k);
-
-    QList<DeployConfigurationFactory *> deployFactories() const;
+    TargetPrivate(Target *t, Kit *k) :
+        m_kit(k),
+        m_buildConfigurationModel(t),
+        m_deployConfigurationModel(t),
+        m_runConfigurationModel(t)
+    { }
 
     bool m_isEnabled = true;
-    QIcon m_icon;
     QIcon m_overlayIcon;
-    QString m_toolTip;
 
     QList<BuildConfiguration *> m_buildConfigurations;
-    BuildConfiguration *m_activeBuildConfiguration = 0;
+    BuildConfiguration *m_activeBuildConfiguration = nullptr;
     QList<DeployConfiguration *> m_deployConfigurations;
-    DeployConfiguration *m_activeDeployConfiguration = 0;
+    DeployConfiguration *m_activeDeployConfiguration = nullptr;
     QList<RunConfiguration *> m_runConfigurations;
-    RunConfiguration* m_activeRunConfiguration = 0;
+    RunConfiguration* m_activeRunConfiguration = nullptr;
     DeploymentData m_deploymentData;
-    BuildTargetInfoList m_appTargets;
+    QList<BuildTargetInfo> m_appTargets;
     QVariantMap m_pluginSettings;
 
-    QPixmap m_connectedPixmap;
-    QPixmap m_readyToUsePixmap;
-    QPixmap m_disconnectedPixmap;
-
     Kit *const m_kit;
+    MacroExpander m_macroExpander;
+
+    ProjectConfigurationModel m_buildConfigurationModel;
+    ProjectConfigurationModel m_deployConfigurationModel;
+    ProjectConfigurationModel m_runConfigurationModel;
 };
 
-TargetPrivate::TargetPrivate(Kit *k) :
-    m_connectedPixmap(QLatin1String(":/projectexplorer/images/DeviceConnected.png")),
-    m_readyToUsePixmap(QLatin1String(":/projectexplorer/images/DeviceReadyToUse.png")),
-    m_disconnectedPixmap(QLatin1String(":/projectexplorer/images/DeviceDisconnected.png")),
-    m_kit(k)
-{ }
 
-QList<DeployConfigurationFactory *> TargetPrivate::deployFactories() const
-{
-    return ExtensionSystem::PluginManager::getObjects<DeployConfigurationFactory>();
-}
-
-Target::Target(Project *project, Kit *k) :
-    ProjectConfiguration(project, k->id()),
-    d(new TargetPrivate(k))
+Target::Target(Project *project, Kit *k, _constructor_tag) :
+    QObject(project),
+    d(std::make_unique<TargetPrivate>(this, k))
 {
     QTC_CHECK(d->m_kit);
     connect(DeviceManager::instance(), &DeviceManager::updated, this, &Target::updateDeviceState);
-
-    setDisplayName(d->m_kit->displayName());
-    setIcon(d->m_kit->icon());
+    connect(project, &Project::parsingFinished, this, [this](bool success) {
+        if (success && this->project() == SessionManager::startupProject()
+                && this == this->project()->activeTarget()) {
+            updateDefaultRunConfigurations();
+        }
+    }, Qt::QueuedConnection); // Must wait for run configs to change their enabled state.
 
     KitManager *km = KitManager::instance();
     connect(km, &KitManager::kitUpdated, this, &Target::handleKitUpdates);
@@ -141,7 +151,6 @@ Target::Target(Project *project, Kit *k) :
             QCoreApplication::translate("ProjectExplorer", "Name of current project"),
             [project] { return project->displayName(); },
             false);
-
 }
 
 Target::~Target()
@@ -149,42 +158,6 @@ Target::~Target()
     qDeleteAll(d->m_buildConfigurations);
     qDeleteAll(d->m_deployConfigurations);
     qDeleteAll(d->m_runConfigurations);
-    delete d;
-}
-
-void Target::changeEnvironment()
-{
-    BuildConfiguration *bc = qobject_cast<BuildConfiguration *>(sender());
-    if (bc == activeBuildConfiguration())
-        emit environmentChanged();
-}
-
-void Target::changeBuildConfigurationEnabled()
-{
-    BuildConfiguration *bc = qobject_cast<BuildConfiguration *>(sender());
-    if (bc == activeBuildConfiguration())
-        emit buildConfigurationEnabledChanged();
-}
-
-void Target::changeDeployConfigurationEnabled()
-{
-    DeployConfiguration *dc = qobject_cast<DeployConfiguration *>(sender());
-    if (dc == activeDeployConfiguration())
-        emit deployConfigurationEnabledChanged();
-}
-
-void Target::changeRunConfigurationEnabled()
-{
-    RunConfiguration *rc = qobject_cast<RunConfiguration *>(sender());
-    if (rc == activeRunConfiguration())
-        emit runConfigurationEnabledChanged();
-}
-
-void Target::onBuildDirectoryChanged()
-{
-    BuildConfiguration *bc = qobject_cast<BuildConfiguration *>(sender());
-    if (bc && activeBuildConfiguration() == bc)
-        emit buildDirectoryChanged();
 }
 
 void Target::handleKitUpdates(Kit *k)
@@ -192,10 +165,10 @@ void Target::handleKitUpdates(Kit *k)
     if (k != d->m_kit)
         return;
 
-    setDisplayName(k->displayName());
-    setIcon(k->icon());
     updateDefaultDeployConfigurations();
     updateDeviceState(); // in case the device changed...
+
+    emit iconChanged();
     emit kitChanged();
 }
 
@@ -206,6 +179,11 @@ void Target::handleKitRemoval(Kit *k)
     project()->removeTarget(this);
 }
 
+bool Target::isActive() const
+{
+    return project()->activeTarget() == this;
+}
+
 Project *Target::project() const
 {
     return static_cast<Project *>(parent());
@@ -216,59 +194,71 @@ Kit *Target::kit() const
     return d->m_kit;
 }
 
-void Target::addBuildConfiguration(BuildConfiguration *configuration)
+Core::Id Target::id() const
 {
-    QTC_ASSERT(configuration && !d->m_buildConfigurations.contains(configuration), return);
-    Q_ASSERT(configuration->target() == this);
+    return d->m_kit->id();
+}
+
+QString Target::displayName() const
+{
+    return d->m_kit->displayName();
+}
+
+QString Target::toolTip() const
+{
+    return d->m_kit->toHtml();
+}
+
+void Target::addBuildConfiguration(BuildConfiguration *bc)
+{
+    QTC_ASSERT(bc && !d->m_buildConfigurations.contains(bc), return);
+    Q_ASSERT(bc->target() == this);
 
     // Check that we don't have a configuration with the same displayName
-    QString configurationDisplayName = configuration->displayName();
+    QString configurationDisplayName = bc->displayName();
     QStringList displayNames = Utils::transform(d->m_buildConfigurations, &BuildConfiguration::displayName);
-    configurationDisplayName = Project::makeUnique(configurationDisplayName, displayNames);
-    if (configurationDisplayName != configuration->displayName()) {
-        if (configuration->usesDefaultDisplayName())
-            configuration->setDefaultDisplayName(configurationDisplayName);
+    configurationDisplayName = Utils::makeUniquelyNumbered(configurationDisplayName, displayNames);
+    if (configurationDisplayName != bc->displayName()) {
+        if (bc->usesDefaultDisplayName())
+            bc->setDefaultDisplayName(configurationDisplayName);
         else
-            configuration->setDisplayName(configurationDisplayName);
+            bc->setDisplayName(configurationDisplayName);
     }
 
     // add it
-    d->m_buildConfigurations.push_back(configuration);
+    d->m_buildConfigurations.push_back(bc);
 
-    emit addedBuildConfiguration(configuration);
-
-    connect(configuration, &BuildConfiguration::environmentChanged,
-            this, &Target::changeEnvironment);
-    connect(configuration, &BuildConfiguration::enabledChanged,
-            this, &Target::changeBuildConfigurationEnabled);
-    connect(configuration, &BuildConfiguration::buildDirectoryChanged,
-            this, &Target::onBuildDirectoryChanged);
+    project()->addedProjectConfiguration(bc);
+    emit addedBuildConfiguration(bc);
+    d->m_buildConfigurationModel.addProjectConfiguration(bc);
 
     if (!activeBuildConfiguration())
-        setActiveBuildConfiguration(configuration);
+        setActiveBuildConfiguration(bc);
 }
 
-bool Target::removeBuildConfiguration(BuildConfiguration *configuration)
+bool Target::removeBuildConfiguration(BuildConfiguration *bc)
 {
     //todo: this might be error prone
-    if (!d->m_buildConfigurations.contains(configuration))
+    if (!d->m_buildConfigurations.contains(bc))
         return false;
 
-    if (BuildManager::isBuilding(configuration))
+    if (BuildManager::isBuilding(bc))
         return false;
 
-    d->m_buildConfigurations.removeOne(configuration);
+    d->m_buildConfigurations.removeOne(bc);
 
-    emit removedBuildConfiguration(configuration);
-
-    if (activeBuildConfiguration() == configuration) {
+    if (activeBuildConfiguration() == bc) {
         if (d->m_buildConfigurations.isEmpty())
-            SessionManager::setActiveBuildConfiguration(this, 0, SetActive::Cascade);
+            SessionManager::setActiveBuildConfiguration(this, nullptr, SetActive::Cascade);
         else
             SessionManager::setActiveBuildConfiguration(this, d->m_buildConfigurations.at(0), SetActive::Cascade);
     }
 
-    delete configuration;
+    emit removedBuildConfiguration(bc);
+    project()->removedProjectConfiguration(bc);
+    d->m_buildConfigurationModel.removeProjectConfiguration(bc);
+
+    delete bc;
     return true;
 }
 
@@ -282,16 +272,14 @@ BuildConfiguration *Target::activeBuildConfiguration() const
     return d->m_activeBuildConfiguration;
 }
 
-void Target::setActiveBuildConfiguration(BuildConfiguration *configuration)
+void Target::setActiveBuildConfiguration(BuildConfiguration *bc)
 {
-    if ((!configuration && d->m_buildConfigurations.isEmpty()) ||
-        (configuration && d->m_buildConfigurations.contains(configuration) &&
-         configuration != d->m_activeBuildConfiguration)) {
-        d->m_activeBuildConfiguration = configuration;
+    if ((!bc && d->m_buildConfigurations.isEmpty()) ||
+        (bc && d->m_buildConfigurations.contains(bc) &&
+         bc != d->m_activeBuildConfiguration)) {
+        d->m_activeBuildConfiguration = bc;
+        project()->activeBuildConfigurationChanged(d->m_activeBuildConfiguration);
         emit activeBuildConfigurationChanged(d->m_activeBuildConfiguration);
-        emit environmentChanged();
-        emit buildConfigurationEnabledChanged();
-        emit buildDirectoryChanged();
     }
 }
 
@@ -300,21 +288,17 @@ void Target::addDeployConfiguration(DeployConfiguration *dc)
     QTC_ASSERT(dc && !d->m_deployConfigurations.contains(dc), return);
     Q_ASSERT(dc->target() == this);
 
-    if (d->deployFactories().isEmpty())
-        return;
-
     // Check that we don't have a configuration with the same displayName
     QString configurationDisplayName = dc->displayName();
     QStringList displayNames = Utils::transform(d->m_deployConfigurations, &DeployConfiguration::displayName);
-    configurationDisplayName = Project::makeUnique(configurationDisplayName, displayNames);
+    configurationDisplayName = Utils::makeUniquelyNumbered(configurationDisplayName, displayNames);
     dc->setDisplayName(configurationDisplayName);
 
     // add it
     d->m_deployConfigurations.push_back(dc);
 
-    connect(dc, &DeployConfiguration::enabledChanged,
-            this, &Target::changeDeployConfigurationEnabled);
-
+    project()->addedProjectConfiguration(dc);
+    d->m_deployConfigurationModel.addProjectConfiguration(dc);
     emit addedDeployConfiguration(dc);
 
     if (!d->m_activeDeployConfiguration)
@@ -333,16 +317,18 @@ bool Target::removeDeployConfiguration(DeployConfiguration *dc)
 
     d->m_deployConfigurations.removeOne(dc);
 
-    emit removedDeployConfiguration(dc);
-
     if (activeDeployConfiguration() == dc) {
         if (d->m_deployConfigurations.isEmpty())
-            SessionManager::setActiveDeployConfiguration(this, 0, SetActive::Cascade);
+            SessionManager::setActiveDeployConfiguration(this, nullptr, SetActive::Cascade);
         else
             SessionManager::setActiveDeployConfiguration(this, d->m_deployConfigurations.at(0),
                                                          SetActive::Cascade);
     }
 
+    project()->removedProjectConfiguration(dc);
+    d->m_deployConfigurationModel.removeProjectConfiguration(dc);
+    emit removedDeployConfiguration(dc);
+
     delete dc;
     return true;
 }
@@ -364,7 +350,6 @@ void Target::setActiveDeployConfiguration(DeployConfiguration *dc)
          dc != d->m_activeDeployConfiguration)) {
         d->m_activeDeployConfiguration = dc;
         emit activeDeployConfigurationChanged(d->m_activeDeployConfiguration);
-        emit deployConfigurationEnabledChanged();
     }
     updateDeviceState();
 }
@@ -374,6 +359,7 @@ void Target::setDeploymentData(const DeploymentData &deploymentData)
     if (d->m_deploymentData != deploymentData) {
         d->m_deploymentData = deploymentData;
         emit deploymentDataChanged();
+        emit applicationTargetsChanged();
     }
 }
 
@@ -382,77 +368,97 @@ DeploymentData Target::deploymentData() const
     return d->m_deploymentData;
 }
 
-void Target::setApplicationTargets(const BuildTargetInfoList &appTargets)
+void Target::setApplicationTargets(const QList<BuildTargetInfo> &appTargets)
 {
-    if (d->m_appTargets != appTargets) {
+    if (Utils::toSet(appTargets) != Utils::toSet(d->m_appTargets)) {
         d->m_appTargets = appTargets;
         emit applicationTargetsChanged();
     }
 }
 
-BuildTargetInfoList Target::applicationTargets() const
+const QList<BuildTargetInfo> Target::applicationTargets() const
 {
     return d->m_appTargets;
 }
 
+BuildTargetInfo Target::buildTarget(const QString &buildKey) const
+{
+    return Utils::findOrDefault(d->m_appTargets, [&buildKey](const BuildTargetInfo &ti) {
+        return ti.buildKey == buildKey;
+    });
+}
+
+QList<ProjectConfiguration *> Target::projectConfigurations() const
+{
+    QList<ProjectConfiguration *> result;
+    result.append(Utils::static_container_cast<ProjectConfiguration *>(buildConfigurations()));
+    result.append(Utils::static_container_cast<ProjectConfiguration *>(deployConfigurations()));
+    result.append(Utils::static_container_cast<ProjectConfiguration *>(runConfigurations()));
+    return result;
+}
+
 QList<RunConfiguration *> Target::runConfigurations() const
 {
     return d->m_runConfigurations;
 }
 
-void Target::addRunConfiguration(RunConfiguration* runConfiguration)
+void Target::addRunConfiguration(RunConfiguration *rc)
 {
-    QTC_ASSERT(runConfiguration && !d->m_runConfigurations.contains(runConfiguration), return);
-    Q_ASSERT(runConfiguration->target() == this);
-    runConfiguration->addExtraAspects();
+    QTC_ASSERT(rc && !d->m_runConfigurations.contains(rc), return);
+    Q_ASSERT(rc->target() == this);
 
     // Check that we don't have a configuration with the same displayName
-    QString configurationDisplayName = runConfiguration->displayName();
-    QStringList displayNames = Utils::transform(d->m_runConfigurations, &RunConfiguration::displayName);
-    configurationDisplayName = Project::makeUnique(configurationDisplayName, displayNames);
-    runConfiguration->setDisplayName(configurationDisplayName);
+    QString configurationDisplayName = rc->displayName();
+    if (!configurationDisplayName.isEmpty()) {
+        QStringList displayNames = Utils::transform(d->m_runConfigurations,
+                                                    &RunConfiguration::displayName);
+        configurationDisplayName = Utils::makeUniquelyNumbered(configurationDisplayName,
+                                                               displayNames);
+        rc->setDisplayName(configurationDisplayName);
+    }
 
-    d->m_runConfigurations.push_back(runConfiguration);
+    d->m_runConfigurations.push_back(rc);
 
-    connect(runConfiguration, &RunConfiguration::enabledChanged,
-            this, &Target::changeRunConfigurationEnabled);
-
-    emit addedRunConfiguration(runConfiguration);
+    project()->addedProjectConfiguration(rc);
+    d->m_runConfigurationModel.addProjectConfiguration(rc);
+    emit addedRunConfiguration(rc);
 
     if (!activeRunConfiguration())
-        setActiveRunConfiguration(runConfiguration);
+        setActiveRunConfiguration(rc);
 }
 
-void Target::removeRunConfiguration(RunConfiguration* runConfiguration)
+void Target::removeRunConfiguration(RunConfiguration *rc)
 {
-    QTC_ASSERT(runConfiguration && d->m_runConfigurations.contains(runConfiguration), return);
+    QTC_ASSERT(rc && d->m_runConfigurations.contains(rc), return);
 
-    d->m_runConfigurations.removeOne(runConfiguration);
+    d->m_runConfigurations.removeOne(rc);
 
-    if (activeRunConfiguration() == runConfiguration) {
+    if (activeRunConfiguration() == rc) {
         if (d->m_runConfigurations.isEmpty())
-            setActiveRunConfiguration(0);
+            setActiveRunConfiguration(nullptr);
         else
             setActiveRunConfiguration(d->m_runConfigurations.at(0));
     }
 
-    emit removedRunConfiguration(runConfiguration);
-    delete runConfiguration;
+    emit removedRunConfiguration(rc);
+    project()->removedProjectConfiguration(rc);
+    d->m_runConfigurationModel.removeProjectConfiguration(rc);
+
+    delete rc;
 }
 
-RunConfiguration* Target::activeRunConfiguration() const
+RunConfiguration *Target::activeRunConfiguration() const
 {
     return d->m_activeRunConfiguration;
 }
 
-void Target::setActiveRunConfiguration(RunConfiguration* configuration)
+void Target::setActiveRunConfiguration(RunConfiguration *rc)
 {
-    if ((!configuration && d->m_runConfigurations.isEmpty()) ||
-        (configuration && d->m_runConfigurations.contains(configuration) &&
-         configuration != d->m_activeRunConfiguration)) {
-        d->m_activeRunConfiguration = configuration;
+    if ((!rc && d->m_runConfigurations.isEmpty()) ||
+        (rc && d->m_runConfigurations.contains(rc) &&
+         rc != d->m_activeRunConfiguration)) {
+        d->m_activeRunConfiguration = rc;
         emit activeRunConfigurationChanged(d->m_activeRunConfiguration);
-        emit runConfigurationEnabledChanged();
     }
     updateDeviceState();
 }
@@ -464,13 +470,7 @@ bool Target::isEnabled() const
 
 QIcon Target::icon() const
 {
-    return d->m_icon;
-}
-
-void Target::setIcon(QIcon icon)
-{
-    d->m_icon = icon;
-    emit iconChanged();
+    return d->m_kit->icon();
 }
 
 QIcon Target::overlayIcon() const
@@ -478,21 +478,16 @@ QIcon Target::overlayIcon() const
     return d->m_overlayIcon;
 }
 
-void Target::setOverlayIcon(QIcon icon)
+void Target::setOverlayIcon(const QIcon &icon)
 {
     d->m_overlayIcon = icon;
     emit overlayIconChanged();
 }
 
-QString Target::toolTip() const
+QString Target::overlayIconToolTip()
 {
-    return d->m_toolTip;
-}
-
-void Target::setToolTip(const QString &text)
-{
-    d->m_toolTip = text;
-    emit toolTipChanged();
+    IDevice::ConstPtr current = DeviceKitAspect::device(kit());
+    return current.isNull() ? QString() : formatDeviceInfo(current->deviceInformation());
 }
 
 QVariantMap Target::toMap() const
@@ -500,7 +495,18 @@ QVariantMap Target::toMap() const
     if (!d->m_kit) // Kit was deleted, target is only around to be copied.
         return QVariantMap();
 
-    QVariantMap map(ProjectConfiguration::toMap());
+    QVariantMap map;
+
+    {
+        // FIXME: For compatibility within the 4.11 cycle, remove this block later.
+        // This is only read by older versions of Creator, but even there not actively used.
+        const char CONFIGURATION_ID_KEY[] = "ProjectExplorer.ProjectConfiguration.Id";
+        const char DISPLAY_NAME_KEY[] = "ProjectExplorer.ProjectConfiguration.DisplayName";
+        const char DEFAULT_DISPLAY_NAME_KEY[] = "ProjectExplorer.ProjectConfiguration.DefaultDisplayName";
+        map.insert(QLatin1String(CONFIGURATION_ID_KEY), id().toSetting());
+        map.insert(QLatin1String(DISPLAY_NAME_KEY), displayName());
+        map.insert(QLatin1String(DEFAULT_DISPLAY_NAME_KEY), displayName());
+    }
 
     const QList<BuildConfiguration *> bcs = buildConfigurations();
     map.insert(QLatin1String(ACTIVE_BC_KEY), bcs.indexOf(d->m_activeBuildConfiguration));
@@ -527,19 +533,15 @@ QVariantMap Target::toMap() const
 
 void Target::updateDefaultBuildConfigurations()
 {
-    IBuildConfigurationFactory *bcFactory = IBuildConfigurationFactory::find(this);
+    BuildConfigurationFactory *bcFactory = BuildConfigurationFactory::find(this);
     if (!bcFactory) {
         qWarning("No build configuration factory found for target id '%s'.", qPrintable(id().toString()));
         return;
     }
-    QList<BuildInfo *> infoList = bcFactory->availableSetups(this->kit(), project()->projectFilePath().toString());
-    foreach (BuildInfo *info, infoList) {
-        BuildConfiguration *bc = bcFactory->create(this, info);
-        if (!bc)
-            continue;
-        addBuildConfiguration(bc);
+    for (const BuildInfo &info : bcFactory->allAvailableSetups(kit(), project()->projectFilePath())) {
+        if (BuildConfiguration *bc = bcFactory->create(this, info))
+            addBuildConfiguration(bc);
     }
-    qDeleteAll(infoList);
 }
 
 void Target::updateDefaultDeployConfigurations()
@@ -552,7 +554,7 @@ void Target::updateDefaultDeployConfigurations()
 
     QList<Core::Id> dcIds;
     foreach (DeployConfigurationFactory *dcFactory, dcFactories)
-        dcIds.append(dcFactory->availableCreationIds(this));
+        dcIds.append(dcFactory->creationId());
 
     QList<DeployConfiguration *> dcList = deployConfigurations();
     QList<Core::Id> toCreate = dcIds;
@@ -566,8 +568,8 @@ void Target::updateDefaultDeployConfigurations()
 
     foreach (Core::Id id, toCreate) {
         foreach (DeployConfigurationFactory *dcFactory, dcFactories) {
-            if (dcFactory->canCreate(this, id)) {
-                DeployConfiguration *dc = dcFactory->create(this, id);
+            if (dcFactory->creationId() == id) {
+                DeployConfiguration *dc = dcFactory->create(this);
                 if (dc) {
                     QTC_CHECK(dc->id() == id);
                     addDeployConfiguration(dc);
@@ -579,8 +581,11 @@ void Target::updateDefaultDeployConfigurations()
 
 void Target::updateDefaultRunConfigurations()
 {
-    QList<IRunConfigurationFactory *> rcFactories = IRunConfigurationFactory::find(this);
-    if (rcFactories.isEmpty()) {
+    // Manual and Auto
+    const QList<RunConfigurationCreationInfo> creators
+            = RunConfigurationFactory::creatorsForTarget(this);
+
+    if (creators.isEmpty()) {
         qWarning("No run configuration factory found for target id '%s'.", qPrintable(id().toString()));
         return;
     }
@@ -596,62 +601,61 @@ void Target::updateDefaultRunConfigurations()
                                [](const RunConfiguration *rc) { return rc->isConfigured(); });
     int configuredCount = existingConfigured.count();
 
-    // find all RC ids that can get created:
-    QList<Core::Id> availableFactoryIds;
-    foreach (IRunConfigurationFactory *rcFactory, rcFactories)
-        availableFactoryIds.append(rcFactory->availableCreationIds(this));
-
-    QList<Core::Id> autoCreateFactoryIds;
-    foreach (IRunConfigurationFactory *rcFactory, rcFactories)
-        autoCreateFactoryIds.append(rcFactory->availableCreationIds(this,
-                                                                    IRunConfigurationFactory::AutoCreate));
-
     // Put outdated RCs into toRemove, do not bother with factories
     // that produce already existing RCs
     QList<RunConfiguration *> toRemove;
-    QList<Core::Id> toIgnore;
+    QList<RunConfigurationCreationInfo> existing;
     foreach (RunConfiguration *rc, existingConfigured) {
-        if (availableFactoryIds.contains(rc->id()))
-            toIgnore.append(rc->id()); // Already there
-        else if (project()->knowsAllBuildExecutables())
-            toRemove << rc;
+        bool present = false;
+        for (const RunConfigurationCreationInfo &item : creators) {
+            QString buildKey = rc->buildKey();
+            if (item.id == rc->id() && item.buildKey == buildKey) {
+                existing.append(item);
+                present = true;
+            }
+        }
+        if (!present && project()->knowsAllBuildExecutables())
+            toRemove.append(rc);
     }
-    foreach (Core::Id i, toIgnore)
-        autoCreateFactoryIds.removeAll(i);
     configuredCount -= toRemove.count();
 
-    // Create new RCs and put them into newConfigured/newUnconfigured
-    foreach (Core::Id id, autoCreateFactoryIds) {
-        IRunConfigurationFactory *factory = Utils::findOrDefault(rcFactories,
-                                                          [this, id] (IRunConfigurationFactory *i) {
-                                                                return i->canCreate(this, id);
-                                                          });
-        if (!factory)
-            continue;
-
-        RunConfiguration *rc = factory->create(this, id);
-        if (!rc)
-            continue;
-        QTC_CHECK(rc->id() == id);
-        if (!rc->isConfigured())
-            newUnconfigured << rc;
-        else
-            newConfigured << rc;
-    }
-    configuredCount += newConfigured.count();
-
-    // Decide what to do with the different categories:
     bool removeExistingUnconfigured = false;
-    if (configuredCount > 0) {
-        // new non-Custom Executable RCs were added
-        removeExistingUnconfigured = true;
-        qDeleteAll(newUnconfigured);
-        newUnconfigured.clear();
-    } else {
-        // no new RCs, use old or new CERCs?
-        if (!existingUnconfigured.isEmpty()) {
+    if (ProjectExplorerPlugin::projectExplorerSettings().automaticallyCreateRunConfigurations) {
+        // Create new "automatic" RCs and put them into newConfigured/newUnconfigured
+        foreach (const RunConfigurationCreationInfo &item, creators) {
+            if (item.creationMode == RunConfigurationCreationInfo::ManualCreationOnly)
+                continue;
+            bool exists = false;
+            for (const RunConfigurationCreationInfo &ex : existing) {
+                if (ex.id == item.id && ex.buildKey == item.buildKey)
+                    exists = true;
+            }
+            if (exists)
+                continue;
+
+            RunConfiguration *rc = item.create(this);
+            if (!rc)
+                continue;
+            QTC_CHECK(rc->id() == item.id);
+            if (!rc->isConfigured())
+                newUnconfigured << rc;
+            else
+                newConfigured << rc;
+        }
+        configuredCount += newConfigured.count();
+
+        // Decide what to do with the different categories:
+        if (configuredCount > 0) {
+            // new non-Custom Executable RCs were added
+            removeExistingUnconfigured = true;
             qDeleteAll(newUnconfigured);
             newUnconfigured.clear();
+        } else {
+            // no new RCs, use old or new CERCs?
+            if (!existingUnconfigured.isEmpty()) {
+                qDeleteAll(newUnconfigured);
+                newUnconfigured.clear();
+            }
         }
     }
 
@@ -675,7 +679,7 @@ void Target::updateDefaultRunConfigurations()
 
     // Make sure a configured RC will be active after we delete the RCs:
     RunConfiguration *active = activeRunConfiguration();
-    if (removalList.contains(active) || !active->isEnabled()) {
+    if (active && (removalList.contains(active) || !active->isEnabled())) {
         RunConfiguration *newConfiguredDefault = newConfigured.isEmpty() ? nullptr : newConfigured.at(0);
 
         RunConfiguration *rc
@@ -716,54 +720,68 @@ void Target::setNamedSettings(const QString &name, const QVariant &value)
         d->m_pluginSettings.insert(name, value);
 }
 
-static QString formatToolTip(const IDevice::DeviceInfo &input)
+QVariant Target::additionalData(Core::Id id) const
 {
-    QStringList lines;
-    foreach (const IDevice::DeviceInfoItem &item, input)
-        lines << QString::fromLatin1("<b>%1:</b> %2").arg(item.key, item.value);
-    return lines.join(QLatin1String("<br>"));
+    return project()->additionalData(id, this);
+}
+
+MakeInstallCommand Target::makeInstallCommand(const QString &installRoot) const
+{
+    return project()->makeInstallCommand(this, installRoot);
+}
+
+MacroExpander *Target::macroExpander() const
+{
+    return &d->m_macroExpander;
+}
+
+ProjectConfigurationModel *Target::buildConfigurationModel() const
+{
+    return &d->m_buildConfigurationModel;
+}
+
+ProjectConfigurationModel *Target::deployConfigurationModel() const
+{
+    return &d->m_deployConfigurationModel;
+}
+
+ProjectConfigurationModel *Target::runConfigurationModel() const
+{
+    return &d->m_runConfigurationModel;
 }
 
 void Target::updateDeviceState()
 {
-    IDevice::ConstPtr current = DeviceKitInformation::device(kit());
+    IDevice::ConstPtr current = DeviceKitAspect::device(kit());
 
-    QPixmap overlay;
+    QIcon overlay;
+    static const QIcon disconnected = Icons::DEVICE_DISCONNECTED_INDICATOR_OVERLAY.icon();
     if (current.isNull()) {
-        overlay = d->m_disconnectedPixmap;
+        overlay = disconnected;
     } else {
         switch (current->deviceState()) {
         case IDevice::DeviceStateUnknown:
-            setOverlayIcon(QIcon());
-            setToolTip(QString());
+            overlay = QIcon();
             return;
-        case IDevice::DeviceReadyToUse:
-            overlay = d->m_readyToUsePixmap;
+        case IDevice::DeviceReadyToUse: {
+            static const QIcon ready = Icons::DEVICE_READY_INDICATOR_OVERLAY.icon();
+            overlay = ready;
             break;
-        case IDevice::DeviceConnected:
-            overlay = d->m_connectedPixmap;
+        }
+        case IDevice::DeviceConnected: {
+            static const QIcon connected = Icons::DEVICE_CONNECTED_INDICATOR_OVERLAY.icon();
+            overlay = connected;
             break;
+        }
         case IDevice::DeviceDisconnected:
-            overlay = d->m_disconnectedPixmap;
+            overlay = disconnected;
             break;
         default:
             break;
         }
     }
 
-    static const int TARGET_OVERLAY_ORIGINAL_SIZE = 32;
-
-    double factor = Core::Constants::TARGET_ICON_SIZE / (double)TARGET_OVERLAY_ORIGINAL_SIZE;
-    QSize overlaySize(overlay.size().width()*factor, overlay.size().height()*factor);
-    QPixmap pixmap(Core::Constants::TARGET_ICON_SIZE, Core::Constants::TARGET_ICON_SIZE);
-    pixmap.fill(Qt::transparent);
-    QPainter painter(&pixmap);
-    painter.drawPixmap(Core::Constants::TARGET_ICON_SIZE - overlaySize.width(),
-                       Core::Constants::TARGET_ICON_SIZE - overlaySize.height(),
-                       overlay.scaled(overlaySize));
-
-    setOverlayIcon(QIcon(pixmap));
-    setToolTip(current.isNull() ? QString() : formatToolTip(current->deviceInformation()));
+    setOverlayIcon(overlay);
 }
 
 void Target::setEnabled(bool enabled)
@@ -777,13 +795,7 @@ void Target::setEnabled(bool enabled)
 
 bool Target::fromMap(const QVariantMap &map)
 {
-    if (!ProjectConfiguration::fromMap(map))
-        return false;
-
-    QTC_ASSERT(d->m_kit == KitManager::find(id()), return false);
-
-    setDisplayName(d->m_kit->displayName()); // Overwrite displayname read from file
-    setDefaultDisplayName(d->m_kit->displayName());
+    QTC_ASSERT(d->m_kit == KitManager::kit(id()), return false);
 
     bool ok;
     int bcCount = map.value(QLatin1String(BC_COUNT_KEY), 0).toInt(&ok);
@@ -800,14 +812,9 @@ bool Target::fromMap(const QVariantMap &map)
         if (!map.contains(key))
             return false;
         const QVariantMap valueMap = map.value(key).toMap();
-        IBuildConfigurationFactory *factory = IBuildConfigurationFactory::find(this, valueMap);
-        if (!factory) {
-            qWarning("No factory found to restore build configuration!");
-            continue;
-        }
-        BuildConfiguration *bc = factory->restore(this, valueMap);
+        BuildConfiguration *bc = BuildConfigurationFactory::restore(this, valueMap);
         if (!bc) {
-            qWarning("Failed '%s' to restore build configuration!", qPrintable(factory->objectName()));
+            qWarning("No factory found to restore build configuration!");
             continue;
         }
         QTC_CHECK(bc->id() == ProjectExplorer::idFromMap(valueMap));
@@ -815,7 +822,7 @@ bool Target::fromMap(const QVariantMap &map)
         if (i == activeConfiguration)
             setActiveBuildConfiguration(bc);
     }
-    if (buildConfigurations().isEmpty() && IBuildConfigurationFactory::find(this))
+    if (buildConfigurations().isEmpty() && BuildConfigurationFactory::find(this))
         return false;
 
     int dcCount = map.value(QLatin1String(DC_COUNT_KEY), 0).toInt(&ok);
@@ -832,18 +839,13 @@ bool Target::fromMap(const QVariantMap &map)
         if (!map.contains(key))
             return false;
         QVariantMap valueMap = map.value(key).toMap();
-        DeployConfigurationFactory *factory = DeployConfigurationFactory::find(this, valueMap);
-        if (!factory) {
+        DeployConfiguration *dc = DeployConfigurationFactory::restore(this, valueMap);
+        if (!dc) {
             Core::Id id = idFromMap(valueMap);
             qWarning("No factory found to restore deployment configuration of id '%s'!",
                      id.isValid() ? qPrintable(id.toString()) : "UNKNOWN");
             continue;
         }
-        DeployConfiguration *dc = factory->restore(this, valueMap);
-        if (!dc) {
-            qWarning("Factory '%s' failed to restore deployment configuration!", qPrintable(factory->objectName()));
-            continue;
-        }
         QTC_CHECK(dc->id() == ProjectExplorer::idFromMap(valueMap));
         addDeployConfiguration(dc);
         if (i == activeConfiguration)
@@ -866,13 +868,13 @@ bool Target::fromMap(const QVariantMap &map)
 
         // Ignore missing RCs: We will just populate them using the default ones.
         QVariantMap valueMap = map.value(key).toMap();
-        IRunConfigurationFactory *factory = IRunConfigurationFactory::find(this, valueMap);
-        if (!factory)
-            continue;
-        RunConfiguration *rc = factory->restore(this, valueMap);
+        RunConfiguration *rc = RunConfigurationFactory::restore(this, valueMap);
         if (!rc)
             continue;
-        QTC_CHECK(rc->id() == ProjectExplorer::idFromMap(valueMap));
+        const Core::Id theIdFromMap = ProjectExplorer::idFromMap(valueMap);
+        if (!theIdFromMap.toString().contains("///::///")) { // Hack for cmake 4.10 -> 4.11
+            QTC_CHECK(rc->id().withSuffix(rc->buildKey()) == theIdFromMap);
+        }
         addRunConfiguration(rc);
         if (i == activeConfiguration)
             setActiveRunConfiguration(rc);
