@@ -42,9 +42,8 @@ namespace Internal {
     class InternalNode;
     class InternalProperty;
 
-    typedef QSharedPointer<InternalNode> InternalNodePointer;
-    typedef QSharedPointer<InternalProperty> InternalPropertyPointer;
-    typedef QWeakPointer<InternalNode> InternalNodeWeakPointer;
+    using InternalNodePointer = QSharedPointer<InternalNode>;
+    using InternalPropertyPointer = QSharedPointer<InternalProperty>;
 }
 class NodeMetaInfo;
 class AbstractProperty;
@@ -60,7 +59,7 @@ class ModelNode;
 
 QMLDESIGNERCORE_EXPORT QList<Internal::InternalNodePointer> toInternalNodeList(const QList<ModelNode> &nodeList);
 
-typedef QList<QPair<PropertyName, QVariant> > PropertyListType;
+using PropertyListType = QList<QPair<PropertyName, QVariant> >;
 
 class QMLDESIGNERCORE_EXPORT  ModelNode
 {
@@ -93,6 +92,7 @@ public:
     ModelNode& operator=(const ModelNode &other);
     TypeName type() const;
     QString simplifiedTypeName() const;
+    QString displayName() const;
     int minorVersion() const;
     int majorVersion() const;
 
@@ -102,10 +102,13 @@ public:
 
     NodeAbstractProperty parentProperty() const;
     void setParentProperty(NodeAbstractProperty parent);
+    void changeType(const TypeName &typeName, int majorVersion, int minorVersion);
     void setParentProperty(const ModelNode &newParentNode, const PropertyName &propertyName);
     bool hasParentProperty() const;
 
     const QList<ModelNode> directSubModelNodes() const;
+    const QList<ModelNode> directSubModelNodesOfType(const TypeName &typeName) const;
+
     const QList<ModelNode> allSubModelNodes() const;
     const QList<ModelNode> allSubModelNodesAndThisNode() const;
     bool hasAnySubModelNodes() const;
@@ -130,6 +133,7 @@ public:
     QList<NodeProperty> nodeProperties() const;
     QList<NodeListProperty> nodeListProperties() const;
     QList<BindingProperty> bindingProperties() const;
+    QList<SignalHandlerProperty> signalProperties() const;
     PropertyNameList propertyNames() const;
 
     bool hasProperties() const;
@@ -172,9 +176,9 @@ public:
     static int variantUserType();
     QVariant toVariant() const;
 
-    QVariant auxiliaryData(const PropertyName &name) const;
+    const QVariant auxiliaryData(const PropertyName &name) const;
     void setAuxiliaryData(const PropertyName &name, const QVariant &data) const;
-    void removeAuxiliaryData(const PropertyName &name);
+    void removeAuxiliaryData(const PropertyName &name) const;
     bool hasAuxiliaryData(const PropertyName &name) const;
     QHash<PropertyName, QVariant> auxiliaryData() const;
 
@@ -189,6 +193,7 @@ public:
 
     bool isComponent() const;
     bool isSubclassOf(const TypeName &typeName, int majorVersion = -1, int minorVersion = -1) const;
+    QIcon typeIcon() const;
 
 private: // functions
     Internal::InternalNodePointer internalNode() const;
